from django.core.exceptions import SuspiciousOperation
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.urls import reverse
from django.test import RequestFactory, TestCase, override_settings, Client, SimpleTestCase

User = get_user_model()


# -----------------------------------------------------------------------
class UserViewsTest(SimpleTestCase):

    def test_webauthn(self):
        self.assertTrue(True)
