# Generated by Django 5.2.9 on 2026-05-04 21:09

from django.db import migrations, models
from console_webauthn.choices import CredentialFormat, DeviceType

class Migration(migrations.Migration):

    dependencies = [
        ("console_webauthn", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="credential",
            name="aaguid",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterModelOptions(
            name="credential",
            options={
                "verbose_name": "credential",
                "verbose_name_plural": "credentials",
            },
        ),
        migrations.AddField(
            model_name="credential",
            name="credential_fmt",
            field=models.TextField(
                choices=CredentialFormat.choices,
                default=CredentialFormat.Packed,
            )),
        migrations.AddField(
            model_name="credential",
            name="device_type",
            field=models.TextField(
                choices=DeviceType.choices,
                default=DeviceType.MultiDevice,
            ),
        ),
        migrations.AddField(
            model_name="credential",
            name="user_verified",
            field=models.BooleanField(default=False),
        ),
    ]
